==> unicore-ucc6-5.0.0-1.src.tar.gz: ucc-core/src/main/java/de/fzj/unicore/ucc/shell/Shell.java <==
package de.fzj.unicore.ucc.shell;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.PrintWriter;

import jline.ConsoleReader;

import org.apache.commons.cli.OptionBuilder;
import org.apache.commons.cli.ParseException;

import de.fzj.unicore.ucc.Command;
import de.fzj.unicore.ucc.UCC;
import de.fzj.unicore.ucc.actions.ActionBase;
import de.fzj.unicore.ucc.helpers.EndProcessingException;

/**
 * a UCC command that starts an interactive session
 * 
 * TODO some more powerful line editing, command history etc.
 * 
 * @author schuller
 */
public class Shell extends ActionBase {

	public static final String OPT_FILE="f";
	public static final String OPT_FILE_LONG="file";

	private File commandFile=null;

	@Override
	public String getCommandGroup() {
		return super.getCommandGroup();
	}

	@Override
	public String getDescription() {
		return "Starts an interactive UCC session";
	}

	@Override
	public String getName() {
		return "shell";
	}

	@Override
	public String getSynopsis() {
		return "Starts an interactive UCC session that allows you to " +
		"multiple run UCC commands.";
	}

	/**
	 *  creates options for the shell command
	 */
	@Override
	@SuppressWarnings("all")
	protected void createOptions() {
		super.createOptions();
		getOptions().addOption(OptionBuilder.withLongOpt(OPT_FILE_LONG)
				.withDescription("Read input from this file instead stdin")
				.withArgName("Commandsfile")
				.hasArg()
				.isRequired(false)
				.create(OPT_FILE)
		);
	}

	@Override
	public void process(){
		super.process();
		String fileName=getOption(OPT_FILE_LONG, OPT_FILE);
		if(fileName!=null){
			commandFile=new File(fileName);
		}
		run();
	}

	public void run(){
		Command.quitAfterPrintingUsage=false;
		ConsoleReader is=null;
		if(commandFile!=null){
			try{
				is=new ConsoleReader(new FileInputStream(commandFile),new PrintWriter(System.out));
			}
			catch(IOException ex){
				error("Error accessing command file.", ex);
				endProcessing(1);
			}
		}
		else{
			try{
				is=new jline.ConsoleReader();
				String[]cmd=UCC.cmds.keySet().toArray(new String[UCC.cmds.keySet().size()]);
				is.addCompletor(new UCCCompletor(cmd));
			}catch(IOException ex){
				error("Can't read from console.", ex);
				endProcessing(1);
			}
		}

		try {
			System.out.println("UCC "+UCC.getVersion()+", http://www.unicore.eu");
			System.out.print("Welcome to the UCC shell. Enter 'help' for a list of commands. Enter 'exit' to quit.");
			while(true){
				System.out.println();
				String s=commandFile!=null?is.readLine():is.readLine("ucc>");
				if(s==null){
					s="exit";
				}
				s=s.trim();
				if("exit".equalsIgnoreCase(s)){
					message("");
					message("Goodbye.");
					return;
				}
				if("help".equalsIgnoreCase(s)){
					UCC.printUsage(false);
					continue;
				}

				//else it is a command
				verbose(s);
				String[] args=s.split(" +");
				if(args.length==0)continue;
				try{
					//check if it is a special command
					if(processSpecial(args)){
						continue;
					}
					
					Command cmd=UCC.initCommand(args,false);
					if(cmd==null){
						message("No such command: "+args[0]);
						continue;
					}
					//inherit properties
					if(verbose)properties.put(OPT_VERBOSE_LONG,"true");
					if(timing)properties.put(OPT_TIMING_LONG,"true");
					cmd.setProperties(properties);
					cmd.process();
					cmd.postProcess();
				}catch(EndProcessingException epe){
					//OK just read next command
				}
				catch(ParseException pex){
					error("Error parsing commandline arguments.",pex);
				}
				catch(Exception ex){
					error("Error processing command", ex);
				}
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	/**
	 * handle special "shell" commands like 'set'
	 * @param args
	 */
	protected boolean processSpecial(String[] args){
		String cmd=args[0];
		if("set".equalsIgnoreCase(cmd)){
			handleSet(args);
			return true;
		}
		
		return false;
	}
	
	/**
	 * handle the "set" command
	 * @param args
	 */
	protected void handleSet(String[] args){
		String cmd=args[0];
		if(!"set".equalsIgnoreCase(cmd))throw new IllegalArgumentException();
		
		if(args.length==1){
			//print properties
			for(Object key: properties.keySet()){
				String val=properties.getProperty(String.valueOf(key));
				if(OPT_PASSWORD_LONG.equals(key)||OPT_TRUSTSTOREPASSWORD_LONG.equals(key)){
					message(key+"="+"*");
				}
				else{
					message(key+"="+val);
				}
			}
		}
		else {
			//set property
			for(int i=1; i<args.length; i++){
				String[] v=args[i].split("=");
				if(v.length!=2)throw new IllegalArgumentException("Expected key=value, got: "+args[i]);
				properties.setProperty(v[0], v[1]);
			}
		}
	}

}

==> unicore-ucc6-6.0.0-0.src.tar.gz: ucc-core/src/main/java/de/fzj/unicore/ucc/shell/Shell.java <==
package de.fzj.unicore.ucc.shell;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.PrintWriter;

import jline.ConsoleReader;

import org.apache.commons.cli.OptionBuilder;
import org.apache.commons.cli.ParseException;

import de.fzj.unicore.ucc.Command;
import de.fzj.unicore.ucc.UCC;
import de.fzj.unicore.ucc.actions.ActionBase;
import de.fzj.unicore.ucc.helpers.EndProcessingException;

/**
 * a UCC command that starts an interactive session
 * 
 * TODO some more powerful line editing, command history etc.
 * 
 * @author schuller
 */
public class Shell extends ActionBase {

	public static final String OPT_FILE="f";
	public static final String OPT_FILE_LONG="file";

	private File commandFile=null;

	@Override
	public String getCommandGroup() {
		return super.getCommandGroup();
	}

	@Override
	public String getDescription() {
		return "start an interactive UCC session";
	}

	@Override
	public String getName() {
		return "shell";
	}

	@Override
	public String getSynopsis() {
		return "Starts an interactive UCC session that allows you to " +
		"multiple run UCC commands.";
	}

	/**
	 *  creates options for the shell command
	 */
	@Override
	@SuppressWarnings("all")
	protected void createOptions() {
		super.createOptions();
		getOptions().addOption(OptionBuilder.withLongOpt(OPT_FILE_LONG)
				.withDescription("Read input from this file instead stdin")
				.withArgName("Commandsfile")
				.hasArg()
				.isRequired(false)
				.create(OPT_FILE)
		);
	}

	@Override
	public void process(){
		super.process();
		String fileName=getOption(OPT_FILE_LONG, OPT_FILE);
		if(fileName!=null){
			commandFile=new File(fileName);
		}
		run();
	}

	public void run(){
		Command.quitAfterPrintingUsage=false;
		ConsoleReader is=null;
		if(commandFile!=null){
			try{
				is=new ConsoleReader(new FileInputStream(commandFile),new PrintWriter(System.out));
			}
			catch(IOException ex){
				error("Error accessing command file.", ex);
				endProcessing(1);
			}
		}
		else{
			try{
				is=new jline.ConsoleReader();
				String[]cmd=UCC.cmds.keySet().toArray(new String[UCC.cmds.keySet().size()]);
				is.addCompletor(new UCCCompletor(cmd));
			}catch(IOException ex){
				error("Can't read from console.", ex);
				endProcessing(1);
			}
		}

		try {
			System.out.println("UCC "+UCC.getVersion()+", http://www.unicore.eu");
			System.out.print("Welcome to the UCC shell. Enter 'help' for a list of commands. Enter 'exit' to quit.");
			while(true){
				System.out.println();
				String s=commandFile!=null?is.readLine():is.readLine("ucc>");
				if(s==null){
					s="exit";
				}
				s=s.trim();
				if("exit".equalsIgnoreCase(s)){
					message("");
					message("Goodbye.");
					return;
				}
				if("help".equalsIgnoreCase(s)){
					UCC.printUsage(false);
					continue;
				}

				//else it is a command
				verbose(s);
				String[] args=s.split(" +");
				if(args.length==0)continue;
				try{
					//check if it is a special command
					if(processSpecial(args)){
						continue;
					}
					
					Command cmd=UCC.initCommand(args,false);
					if(cmd==null){
						message("No such command: "+args[0]);
						continue;
					}
					//inherit properties
					if(verbose)properties.put(OPT_VERBOSE_LONG,"true");
					if(timing)properties.put(OPT_TIMING_LONG,"true");
					cmd.setProperties(properties);
					cmd.process();
					cmd.postProcess();
				}catch(EndProcessingException epe){
					//OK just read next command
				}
				catch(ParseException pex){
					error("Error parsing commandline arguments.",pex);
				}
				catch(Exception ex){
					error("Error processing command", ex);
				}
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	/**
	 * handle special "shell" commands like 'set'
	 * @param args
	 */
	protected boolean processSpecial(String[] args){
		String cmd=args[0];
		if("set".equalsIgnoreCase(cmd)){
			handleSet(args);
			return true;
		}
		if("unset".equalsIgnoreCase(cmd)){
			handleUnset(args);
			return true;
		}
		return false;
	}
	
	/**
	 * handle the "set" command
	 * @param args
	 */
	protected void handleSet(String[] args){
		String cmd=args[0];
		if(!"set".equalsIgnoreCase(cmd))throw new IllegalArgumentException();
		
		if(args.length==1){
			//print properties
			for(Object keyObj: properties.keySet()){
				String key=String.valueOf(keyObj);
				String val=properties.getProperty(key);
				//naive, but should work rather well :-)
				if(key.toLowerCase().contains("password")){
					message(key+"="+"*");
				}
				else{
					message(key+"="+val);
				}
			}
		}
		else {
			//set property
			for(int i=1; i<args.length; i++){
				String[] v=args[i].split("=",2);
				if(v.length!=2)throw new IllegalArgumentException("Expected key=value, got: "+args[i]);
				properties.setProperty(v[0], v[1]);
			}
		}
	}

	/**
	 * handle the "unset" command
	 * @param args
	 */
	protected void handleUnset(String[] args){
		String cmd=args[0];
		if(!"unset".equalsIgnoreCase(cmd))throw new IllegalArgumentException();
		for(int i=1; i<args.length;i++){
			properties.remove(args[i]);
		}
	}
	
	@Override
	protected boolean requireRegistry() {
		return false;
	}

}
